//#include	<Pa.h>
//#include	<Paerr.h>
#include	"include\Pa.h"
#include	"include\Paerr.h"
#include	<stdio.h>

#define	M_PI	(float)3.14159
#define	r2d		(float)180/M_PI
#define d2r		(float)M_PI/180

void main(void){

//=== System setting ===	
	pa_ini_sys();			// Initialization of PA-Library
	pa_opn_arm( ARM0 );		// select control ( ARM0~ARM15 )
	pa_sta_sim( ARM0 );		// select simulation

//	pa_sta_log( ARM0 , 40 , 120 );
/*========================================================================
 * FUNCTION :
 *   To start logger for arm infomation
 * SYMTAX :
 *   long pa_sta_log(ARM armno,long cyc,long sec)
 *       armno   arm No.( ARM0~ARM15 )
 *       cyc     log interval[msec]
 *       sec     log time [sec]
 *
 *========================================================================*/



//=== pa_exe_axs : Each axis angle control ===
	ANGLE	ang;			
	ang.s1 = 180 * d2r;		
	ang.s2 =  15 * d2r;
	ang.e1 =  30 * d2r;
	ang.w1 =  45 * d2r;
	pa_exe_axs( ARM0 , S2|E1|W1 , &ang , WM_WAIT );


	printf("pa_exe_axs : OK!\n");



//=== pa_get_agl : Load arm current each axis values ===
	pa_get_agl( ARM0 , &ang );
	printf("	ANGLE[deg] =[ %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f ]\n",ang.s1*r2d,ang.s2*r2d,ang.s3*r2d,ang.e1*r2d,ang.e2*r2d,ang.w1*r2d,ang.w2*r2d);

	printf("pa_get_agl : OK!\n");



//=== pa_get_xyz : Load arm current tip pasition ===
	VECTOR	vec;			
	pa_get_xyz( ARM0 , vec );
	printf("	[ X,Y,Z ] =[ %6.1f %6.1f %6.1f ]\n",vec[0],vec[1],vec[2]);

	printf("pa_get_xyz : OK!\n");



//=== pa_mov_XYZ : Position deviation control in robot coordinates ===
	REAL32	X,Y,Z;				
	X =   0;
	Y = 150-vec[1];				
	Z = 700-vec[2];
	pa_mov_XYZ( ARM0 , X , Y , Z , WM_WAIT );	

	pa_get_xyz( ARM0 , vec );
	printf("	[ X,Y,Z ] =[ %6.1f %6.1f %6.1f ]\n",vec[0],vec[1],vec[2]);

	printf("pa_mov_XYZ : OK!\n");



//=== pa_mov_xyz : Position deviation control in arm tip coordinates ===
	X =   0;					
	Y =   0;					
	Z =(float) 701.5-vec[0];	
	pa_mov_xyz( ARM0 , X , Y , Z , WM_WAIT );
	pa_get_xyz( ARM0 , vec );
	printf("	[ X,Y,Z ] =[ %6.1f %6.1f %6.1f ]\n",vec[0],vec[1],vec[2]);

	printf("pa_mov_xyz : OK!\n");



//=== pa_get_noa : Load arm current orientation matrix ===
	MATRIX	noap;
	pa_get_noa(ARM0,noap);
	printf("	           | %6.1f %6.1f %6.1f %6.1f |\n",noap[0][0],noap[0][1],noap[0][2],noap[0][3]);
	printf("	noap[3][4]=| %6.1f %6.1f %6.1f %6.1f |\n",noap[1][0],noap[1][1],noap[1][2],noap[1][3]);
	printf("	           | %6.1f %6.1f %6.1f %6.1f |\n",noap[2][0],noap[2][1],noap[2][2],noap[2][3]);

	printf("pa_get_noa : OK!\n");



//error check(1)
//=== pa_mov_mat : Arm tip position/orientation absolute position control ===
	ERR		err;		
	MATRIX	mat;		
	ANGLE	an;			
	mat[0][0] = 0.0;	// nx 
	mat[1][0] = 0.0;	// ny 
	mat[2][0] =-1.0;	// nz 
	mat[0][1] = 0.0;	// ox 
	mat[1][1] = 1.0;	// oy 
	mat[2][1] = 0.0;	// oz 
	mat[0][2] = 1.0;	// ax 
	mat[1][2] = 0.0;	// ay 
	mat[2][2] = 0.0;	// az 
	mat[0][3] = 701.5;	// px 
	mat[1][3] =-150.0;	// py 
	mat[2][3] = 400.0;	// pz 
	an.s1 =  0 * d2r;
	an.s2 =  0 * d2r;
	an.s3 =  0 * d2r;
	an.e1 = 90 * d2r;	
	an.e2 =  0 * d2r;
	an.w1 =  0 * d2r;
	an.w2 =  0 * d2r;

	printf("pa_mov_mat : ");
	err = pa_mov_mat( ARM0 , MM_XYZ , mat , &an , WM_WAIT );
	if(err!=ERR_OK){
		printf("Error! (%d)\n",err);	
		err = 0;
	}else{
		printf("OK!\n");				
	}



//=== pa_mov_XYZ : Position deviation control in robot coordinate ===
	X =-100.0;
	Y =   0.0;
	Z =   0.0;
	pa_mov_XYZ( ARM0 , X , Y , Z , WM_WAIT );

	printf("pa_mov_XYZ : OK!\n");



//error check(2)
//=== pa_exe_axs : Each axis angle control ===
	int		cnt0,cnt1;				

	ang.s1 = -60 * d2r;
	ang.s3 =   0 * d2r;
	ang.e2 =   0 * d2r;
	ang.w2 =   0 * d2r;

	pa_get_cnt( ARM0 , &cnt0 );				
	err = pa_exe_axs( ARM0 , S1|S3|E2|W2 , &ang , WM_NOWAIT );	
	while(1){
		if( ( err=pa_get_cnt( ARM0 , &cnt1 ) ) != ERR_OK ){
			printf("pa_exe_axs :Error! (%d)\n",err);	
			err = 0;
			break;
		}else if( cnt1 != cnt0 ){
			printf("pa_exe_axs : OK!\n");				
			break;
		}else{
			pa_get_xyz( ARM0 , vec );					//Each axis angle control
			pa_get_agl( ARM0 , &ang );					//Load arm current each axis values
			printf("	[ X,Y,Z ] =[ %6.1f %6.1f %6.1f ]\n",vec[0],vec[1],vec[2]);
			printf("	ANGLE[deg] =[ %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f ]\n",ang.s1*r2d,ang.s2*r2d,ang.s3*r2d,ang.e1*r2d,ang.e2*r2d,ang.w1*r2d,ang.w2*r2d);
		}
	}



//===Each axis motin control function ===
	pa_exe_hom( ARM0 , WM_WAIT );	// Each axis contorol to basic posture
	printf("pa_exe_hom : OK!\n");
	pa_exe_saf( ARM0 , WM_WAIT );	// Each axis cotrol to escape posture
	printf("pa_exe_saf : OK!\n");
	pa_exe_esc( ARM0 , WM_WAIT );	// Each aaxis control to safe posture
	printf("pa_exe_esc : OK!\n");
	pa_exe_hom( ARM0 , WM_WAIT );
	printf("pa_exe_hom : OK!\n");



//	pa_end_log( ARM0 );
/*========================================================================
 * FUNCTION :
 *    To end logger for arm infomation and make a "kinema0.log"
 * SYNTAX :
 *    llong pa_end_log(ARM armno)
 *        armno   arm No.( ARM0~ARM15 )
 *========================================================================*/



//=== Arm condition control function ===
	pa_ext_arm( ARM0 );		// Simulation control end("pa_ext_arm" serves to substitute for "pa_ext_sim")
	pa_cls_arm( ARM0 );		// Close arm(disengage control arm)
	pa_ter_sys();			// Termination of PA-Library
}