/*
 *	pa.h -- Definitions for Parsonal Arm PA10
 *
 *	Copyright (c) Mitsubisgi Heavy Industries, 1993
 *
 */

#ifndef _PA_H_
#define _PA_H_

#define	_PA_	10

/*
 * Holy Oracle for MS-DOS C compilers
 */
#if defined(__TURBOC__)
#  if __STDC__
#    define _Pascal
#  else
#    define _Pascal pascal
#  endif
#  define __PASCAL _Pascal
#elif defined(LSI_C)
#  define __PASCAL
#elif defined(LATTICE)
#  define __PASCAL pascal
#elif defined(_MSC_VER) && _MSC_VER >= 600
#  define __PASCAL _fastcall
#else /* Microsoft C 5.1, Quick C */
#  ifndef NO_EXT_KEYS
#    define _PASCAL pascal
#  else
#    define _PASCAL
#  endif
#  define __PASCAL _PASCAL
#endif


/*========================================================================
 *  Datatype of PA10                                                             
 =========================================================================*/
/* 
 *	@@@@@@@@@@@@@@@@@@@@@
 */
typedef char	CHAR;			
typedef short	INT16;			
typedef int	INT32;			
typedef float	REAL32;			  
typedef double	REAL64;			

typedef unsigned char	UCHAR;
typedef unsigned short 	UINT16;
typedef unsigned long 	UINT32;

typedef CHAR*	STRING;			

/* 
 * 	Data types with specific significance
 */
typedef	INT32	ERR;			/* Error code			*/
typedef REAL32 	MATRIX[3][4];	/* 3~4 matrix indicating the tip position/orientation, etc.	*/
typedef REAL32 	NOAMAT[3][3];	/* 3~3 matrix indicating the tip orientation,			*/
typedef REAL32 	VECTOR[3];		/* Tip position vector, etc.				*/

/* 
 *	Data types with@Function@@@@@@@@@@@@@@@
 */
#define	WM_WAIT		0
#define	WM_NOWAIT	1
/* 
 * @Teach data structure 
 */
typedef struct {
	INT32  s1;
	INT32  s2;
	INT32  s3;
	INT32  e1;
	INT32  e2;
	INT32  w1;
	INT32  w2;
	INT32   speed;
	INT32   rotate;
	INT32   type;
	INT32   fifo;
	INT32   pattarn;
	INT32   time;
	INT32   number;
	INT32   tool0;
	INT32   tool1;
}PNTDAT, *PNTDATP;

/* 
 *  Axis data structure
 */
typedef struct {
	REAL32 s1;
	REAL32 s2;
	REAL32 s3;
	REAL32 e1;
	REAL32 e2;
	REAL32 w1;
	REAL32 w2;
}ANGLE, *ANGLEP;

/* 
 * Arm Status Structure
 */
typedef struct {
	INT32		remote;
	INT32		count;
	INT32		error;
	ANGLE		angle;
	MATRIX		noap;
	REAL32		ypr[3];
}ARMSTATUS, *ARMSTATUSP;

/* 
 * Digital I/O Sstructure
 */
typedef struct {
	UCHAR		io0;	/*DIO 1ch*/
	UCHAR		io1;	/*DIO 2ch*/
	UCHAR		io2;	/*DIO 3ch*/
	UCHAR		io3;	/*DIO 4ch*/
}DIOSTATUS, *DIOSTATUSP;

/* 
 *  Parameter Structure
 */
typedef struct {
	REAL32	rezl;			/* Resolver resolution			*/
	INT32	pul[7];			/* Position limiteri{j		*/
	INT32	pdl[7];			/* Position limiteri|j		*/
	INT32	vel[7 + 2];		/* Velocity limiter			*/
	INT32	dev[7 + 2];		/* Default velocity			*/
	INT32	pg1[7];			/* Position control gainP		*/
	INT32	pg2[7];			/* Position control gainQ		*/
	INT32	vg1[7];			/* Velocity control gain		*/
	INT32	tg1[7];			/* Force control gain			*/
	INT32	pcm[7];			/* position control selection matrix	*/
	INT32	fcm[7];			/* Force control selection matrix  	*/
	INT32	arl[7];			/* Arm length				*/
	INT32	arg[7];			/* Axis gravity center position		*/
	INT32	arw[7];			/* Axis weight				*/
	INT32	rfp[7];			/* Home position recovery target value	*/    			
	INT32	rsp[7];			/* Escape point recovery target value	*/    			
//	INT32	rop[7];			/* Other point recovery target value    */  		@@@  
	INT32	tol[7];			/* Tool parameter			*/
	INT32	dmy[7];			/* (Not available)		*/
	INT32	spa[7];			/* Spare				*/
} PARAM, *PARAMP;

/* 
 * Arm classification
 */
typedef UINT32  ARM;
#define	ARM0	(ARM)0
#define	ARM1	(ARM)1
#define	ARM2	(ARM)2
#define	ARM3	(ARM)3
#define	ARM4	(ARM)4
#define	ARM5	(ARM)5
#define	ARM6	(ARM)6
#define	ARM7	(ARM)7
#define	ARM8	(ARM)8
#define	ARM9	(ARM)9
#define	ARM10	(ARM)10
#define	ARM11	(ARM)11
#define	ARM12	(ARM)12
#define	ARM13	(ARM)13
#define	ARM14	(ARM)14
#define	ARM15	(ARM)15

/* 
 * Axis classification
 */
typedef UINT32	AXIS;
#define	S1	(AXIS)0x01
#define	S2	(AXIS)0x02
#define	S3	(AXIS)0x04
#define	E1	(AXIS)0x08
#define	E2	(AXIS)0x10
#define	W1	(AXIS)0x20
#define	W2	(AXIS)0x40

#define	AXISALL		(S1|S2|S3|E1|E2|W1|W2)
#define	ALLAXIS		(S1|S2|S3|E1|E2|W1|W2)
#define	LOCKAXIS_S1	(S2|S3|E1|E2|W1|W2)
#define	LOCKAXIS_S3	(S1|S2|E1|E2|W1|W2)

/*
 *  Servo driver classification
 */
typedef UINT32	DRIVER;
#define	DRV1	(DRIVER)0
#define	DRV2	(DRIVER)1
#define	DRV3	(DRIVER)2
#define	DRV4	(DRIVER)3

/*	
 *	Playback motion classification
 */
typedef UINT32	PLAYBACK;
#define	PB_FORES	(PLAYBACK)0
#define	PB_BACKS	(PLAYBACK)1
#define	PB_FORE		(PLAYBACK)2
#define	PB_BACK		(PLAYBACK)3

/*
 *	Teach data deletion operation classification
 */
typedef UINT32	PNTDEL;
#define	PD_CUR	(PNTDEL)0x7500		/*Current point teach data deletion		*/
#define	PD_FORE	(PNTDEL)0x7501		/*Previous current point teach data deletion	*/
#define	PD_ALL	(PNTDEL)0x7502		/*All active teach data deletion		*/

/*
 *	Teach data attribution alteration classification
 */
typedef UINT32	PNTATTR;
#define	PA_CHGVEL	(PNTATTR)0x7300
#define	PA_CHGWAIT	(PNTATTR)0x7301
#define	PA_VELPTN	(PNTATTR)0x7302		/* Velocity interpolation	 	*/
#define	PA_ROTVEL	(PNTATTR)0x7303		/* Rotational velocity alteration 	*/

/*
 *	Teach data type classification
 */
typedef UINT32	PNTTYPE;
#define	PT_CP		(PNTTYPE)0x7100
#define	PT_PTP		(PNTTYPE)0x7101
#define	PT_BCP		(PNTTYPE)0x7102
#define	PT_BPTP		(PNTTYPE)0x7103
#define	PT_ARC1		(PNTTYPE)0x7104
#define	PT_ARC2		(PNTTYPE)0x7105
#define	PT_ARC3		(PNTTYPE)0x7106
#define	PT_CIR1		(PNTTYPE)0x7107
#define	PT_CIR2		(PNTTYPE)0x7108
#define	PT_CIR3		(PNTTYPE)0x7109
#define	PT_AXS		(PNTTYPE)0x710a
#define	PT_BAXS		(PNTTYPE)0x710b

/*
 *	Teach data pointer operation classification
 */
typedef UINT32	PNTMOVE;
#define	PM_TOP	(PNTMOVE)0x7100		/* Moves pointer to top.						        */
#define	PM_NEXT	(PNTMOVE)0x7101		/* Pointer forward, once.							*/
#define	PM_PRIV	(PNTMOVE)0x7102		/* Pointer backward, once.							*/
#define	PM_BTM	(PNTMOVE)0x7103		/* Moves pointer to bottom.							*/
#define	PM_JMP	(PNTMOVE)0x7104		/* Moves pointer to designated number.						*/
#define	PM_CIR	(PNTMOVE)0x7105		/* Circle teach point searched,moving pointer to teach point found first. 	*/
#define	PM_ARC	(PNTMOVE)0x7106		/* Arc teach point searched, movingpointer to teach point found first	@	*/

/*
 *	Default velocity alteration classification
 */
typedef UINT32	VELTYPE;
#define	VT_ONEVEL	(VELTYPE)0	/* Each axis default velocity alteration */
#define	VT_XYZVEL	(VELTYPE)1	/* Tip position default velocity alteration  */
#define	VT_YPRVEL	(VELTYPE)2	/* Tip orientation default velocity alteration  */

/*
 *	Velocity control mode classification
 */
typedef UINT32	VELMODE;
#define	VM_XYZ		(VELMODE)0x200
#define	VM_YPR		(VELMODE)0x201
#define	VM_xyz		(VELMODE)0x202
#define	VM_ypr		(VELMODE)0x203
#define	VM_ONE		(VELMODE)0x204
#define	VM_XYZYPR	(VELMODE)0x205
#define	VM_xyzypr	(VELMODE)0x206

/*
 *   Redundant axis control mode classification   
 */
typedef UINT32	JOUMODE;
#define	JM_SET		(JOUMODE)0X345
#define	JM_RESET	(JOUMODE)0X346
#define	JM_VSET		(JOUMODE)0X347
#define	JM_ON		(JOUMODE)0X348
#define	JM_OFF		(JOUMODE)0X349
#define	JM_S3ON		(JOUMODE)0X34a
#define	JM_S3DIV	(JOUMODE)0X34b
#define	JM_S3HOLD	(JOUMODE)0X34c

/*
 *	Redundant axis control mode classification
 */
typedef UINT32	JOUTYPE;
#define	JT_RIGHT	(JOUTYPE)1
#define	JT_HOLD		(JOUTYPE)0
#define	JT_LEFT		(JOUTYPE)-1

/*
 *	
 */
typedef UINT32	MOVEMODE;
#define	MM_XYZ		(MOVEMODE)0x5680
#define	MM_NOA		(MOVEMODE)0x5681
#define	MM_XYZNOA	(MOVEMODE)0x5682

/*
 *	
 */
typedef UINT32	DIRECTMODE;
#define	DM_STOP		(DIRECTMODE)0
#define	DM_START	(DIRECTMODE)1

#define	ARM_STANDING	 1		/* Floor mounted 		*/
#define	ARM_HANGING		-1	/* Suspending from ceiling 	*/

/*
 *	DIO port numbers
 */
typedef UINT32	DIOPORT;
#define	DP_PORT0	(DIOPORT)0
#define	DP_PORT1	(DIOPORT)1
#define	DP_PORT2	(DIOPORT)2
#define	DP_PORT3	(DIOPORT)3
#define	DPO_PORT0	(DIOPORT)4
#define	DPO_PORT1	(DIOPORT)5
#define	DPO_PORT2	(DIOPORT)6
#define	DPO_PORT3	(DIOPORT)7
#define	DPX_PORT0	(DIOPORT)8
#define	DPX_PORT1	(DIOPORT)9
#define	DPX_PORT2	(DIOPORT)10
#define	DPX_PORT3	(DIOPORT)11

/*
 *	DIO channel numbers
 */
typedef UINT32	DIOCH;
#define	DC_CH1	(DIOCH)0
#define	DC_CH2	(DIOCH)1
#define	DC_CH3	(DIOCH)2
#define	DC_CH4	(DIOCH)3
#define	DC_CH5	(DIOCH)4
#define	DC_CH6	(DIOCH)5
#define	DC_CH7	(DIOCH)6
#define	DC_CH8	(DIOCH)7
/*
 *    Sensor correction coordinate classification
 */
typedef UINT32		TRANSMODE;
#define	MODE_xyz	(TRANSMODE)0x01
#define	MODEIxyz	(TRANSMODE)0x02
#define	MODE_XYZ	(TRANSMODE)0x04
#define	MODEIXYZ	(TRANSMODE)0x08
#define	MODE_wave	(TRANSMODE)0x10
#define	MODEIwave	(TRANSMODE)0x20

/*
 *	Teach point attribute designation
 */
typedef UINT32	PNTID;
#define	PA_SETID	(PNTID)0x7304

/*
 *	Circle & arc teach point number designation
 */
typedef UINT32	PNTNO;
#define	PN_1 	(PNTNO)1
#define	PN_2 	(PNTNO)2
#define	PN_3 	(PNTNO)3

/*
 *	Sensor correction data structure
 */
typedef struct {
	INT32	Enable;			/* Designation bit 							*/
	REAL32	_xyz[3];		/* Mechanical interface coordinate absolute deviation correction value 	*/
	REAL32	Ixyz[3];		/* Mechanical interface coordinate relative deviation correction value 	*/
	REAL32	_XYZ[3];		/* Base coordinate absolute deviation correction value 			*/
	REAL32	IXYZ[3];		/* Base coordinate relative deviation correction value 			*/
	REAL32	_wave[3];		/* Trajectory coordinate absolute deviation correction value 		*/
	REAL32	Iwave[3];		/* Trajectory coordinate relative deviation correction value 		*/
} TRANSMAT, *TRANSMATP;

/*
 *	other functon                                              
 */
typedef struct {
	INT32	limit;		
	INT32	pitch;			
	INT32	depth;			
	INT32	width;			
	REAL32	angle;			
} WAVATTR;

/* 
 * Arm target value structure 
 */
typedef struct {
	ANGLE		angle;
	MATRIX		noap;
	REAL32		ypr[3];
} ARMTARGET, *ARMTARGETP;


/*===========================================================================
	 function prototype declaration           
===========================================================================*/
#define	DllExport	__declspec(dllexport)

#ifdef __cplusplus
extern  "C" {
#endif


/*
 *	System Setting & Initialization Function
 */
DllExport ERR __stdcall pa_ini_sys(void);	/* PA library initialization	*/
DllExport ERR __stdcall pa_ter_sys(void);	/* PA library termination	*/

/*
 *	Arm status control function@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@      @@
 */
DllExport ERR __stdcall pa_opn_arm(ARM);	/* Open arm (control arm selection)				*/
DllExport ERR __stdcall pa_cls_arm(ARM);	/* Close arm (control arm separation)				*/

DllExport ERR __stdcall pa_sta_arm(ARM);	/* Controller operation start (Servo driver communication start) */
DllExport ERR __stdcall pa_ext_arm(ARM);	/* Controller operation end (Servo driver communication end)	*/

DllExport ERR __stdcall pa_stp_arm(ARM,INT32);	/* Arm brake-stop			*/
DllExport ERR __stdcall pa_sus_arm(ARM,INT32);	/* Arm temporarily stop			*/
DllExport ERR __stdcall pa_rsm_arm(ARM,INT32);	/* Arm temporarily-stop-release	 	*/

DllExport ERR __stdcall pa_exe_axs(ARM,AXIS,ANGLEP,INT32);/* Axis angle control	*/
DllExport ERR __stdcall pa_exe_edr(ARM,AXIS,ANGLEP,INT32);               

DllExport ERR __stdcall pa_exe_hom(ARM,INT32);	/* Axis angle control to home position		*/
DllExport ERR __stdcall pa_exe_esc(ARM,INT32);	/* Axis angle control to escape position	*/
DllExport ERR __stdcall pa_exe_saf(ARM,INT32);	/* Axis angle control to safety position        */

DllExport ERR __stdcall pa_mov_XYZ(ARM,REAL32,REAL32,REAL32,INT32);
DllExport ERR __stdcall pa_mov_YPR(ARM,REAL32,REAL32,REAL32,INT32);
DllExport ERR __stdcall pa_mov_xyz(ARM,REAL32,REAL32,REAL32,INT32);
DllExport ERR __stdcall pa_mov_ypr(ARM,REAL32,REAL32,REAL32,INT32);

DllExport ERR __stdcall pa_mov_mat(ARM,MOVEMODE,MATRIX,ANGLEP,INT32);

/*
 *	Function on teach point operation & playback control
 */
DllExport ERR __stdcall pa_chg_pnt(ARM,PNTMOVE,INT32);
DllExport ERR __stdcall pa_add_pnt(ARM,PNTTYPE);
DllExport ERR __stdcall pa_del_pnt(ARM,PNTDEL);
DllExport ERR __stdcall pa_rpl_pnt(ARM);
DllExport ERR __stdcall pa_set_pnt(ARM,PNTATTR,INT32);

DllExport ERR __stdcall pa_mov_pnt(ARM,INT32);
DllExport ERR __stdcall pa_ply_pnt(ARM,PLAYBACK,INT32);
DllExport ERR __stdcall pa_lod_pnt(ARM,STRING);
DllExport ERR __stdcall pa_sav_pnt(ARM,STRING);

DllExport ERR __stdcall pa_vel_pnt(ARM,REAL32);	/* Playback control velocity coefficient alteration	*/

/*
 *	Velocity control function
 */
DllExport ERR __stdcall pa_mod_vel(ARM,VELMODE,AXIS);	/*  Velocity control mode setting   */	
DllExport ERR __stdcall pa_odr_vel(ARM,REAL32[]);	/*  Velocity control data set  */
/*
 *	Redundant axis control function
 */
DllExport ERR __stdcall pa_mod_jou(ARM,JOUMODE);
DllExport ERR __stdcall pa_odr_jou(ARM,JOUTYPE);

/*
 *	Tip absolute position/orientation, axis real-time control function
 */
DllExport ERR __stdcall pa_mod_dpd(ARM);
DllExport ERR __stdcall pa_odr_dpd(ARM,MATRIX,ANGLEP);

/*
 *	Direct control function
 */
DllExport ERR __stdcall pa_wet_ded(ARM,AXIS);
DllExport ERR __stdcall pa_get_amp(ARM);
DllExport ERR __stdcall pa_mod_dir(ARM,DIRECTMODE);

/*
 *	Function on position setting/definition
 */
DllExport ERR __stdcall pa_set_hom(ARM,ANGLEP);
DllExport ERR __stdcall pa_set_esc(ARM,ANGLEP);
DllExport ERR __stdcall pa_set_saf(ARM,ANGLEP);

DllExport ERR __stdcall pa_def_hom(ARM);
DllExport ERR __stdcall pa_def_esc(ARM);
DllExport ERR __stdcall pa_def_saf(ARM);

/*
 *	Function on coordinate conversion matrix
 */
DllExport ERR __stdcall pa_set_mtx(ARM,MATRIX);

/*
 *	Arm status information loading function
 */
DllExport ERR __stdcall pa_get_sts(ARM,ARMSTATUSP);
DllExport ERR __stdcall pa_get_cnt(ARM,INT32*);
DllExport ERR __stdcall pa_get_err(ARM,INT32*);
DllExport ERR __stdcall pa_get_agl(ARM,ANGLEP);
DllExport ERR __stdcall pa_get_xyz(ARM,VECTOR);
DllExport ERR __stdcall pa_get_noa(ARM,MATRIX);
DllExport ERR __stdcall pa_get_ypr(ARM,VECTOR);
DllExport ERR __stdcall pa_get_prm(ARM,PARAMP);
DllExport ERR __stdcall pa_get_pnt(ARM,PNTDATP);
DllExport ERR __stdcall pa_get_cur(ARM,INT32*);
DllExport ERR __stdcall pa_get_num(ARM,INT32*);

/*
 *	Digital input/output function
 */
DllExport ERR __stdcall pa_inp_dio(ARM,DIOSTATUSP);
DllExport ERR __stdcall pa_oup_dio(ARM,DIOSTATUSP);
DllExport ERR __stdcall pa_get_dio(ARM,DIOPORT,DIOCH,UCHAR*);
DllExport ERR __stdcall pa_set_dio(ARM,DIOPORT,DIOCH);
DllExport ERR __stdcall pa_rst_dio(ARM,DIOPORT,DIOCH);

/*
 *	Function on parameter
 */
DllExport ERR __stdcall pa_set_tol(ARM,INT32,INT32,INT32,INT32);/* Tool information setting	*/
DllExport ERR __stdcall pa_set_vel(ARM,VELTYPE,REAL32*);/* Default velocity alteration 		*/

DllExport ERR __stdcall pa_lod_ctl(ARM,STRING);		/* loading parameter to controller 	*/
DllExport ERR __stdcall pa_sav_ctl(ARM,STRING);		

//DllExport ERR __stdcall pa_lod_def(ARM);	@@@@@@@
DllExport ERR __stdcall pa_sav_def(ARM);	

/*
 *	Other functions
 */
DllExport ERR __stdcall pa_map_ctl(ARM);	/* Shared area mapping with controller	*/
DllExport INT16 __stdcall pa_fsh_chk(ARM);	/* Waiting for control command processing completion	*/
DllExport ERR __stdcall pa_req_ctl(ARM,INT32);	/* Issuing command setting intrusion to controller*/
DllExport ERR __stdcall pa_rst_ctl(ARM);	/* Arm error information reset			*/

DllExport ERR __stdcall	pa_err_mes(ERR,STRING);	/* Error message acquisition		*/

DllExport ERR __stdcall	pa_clr_log(ARM);	
DllExport ERR __stdcall	pa_sav_log(ARM,STRING);	

/*
 *	Initialization Function
 */
DllExport ERR __stdcall pa_sta_sim(ARM);	/* Simulation control start (simulation communication start)	 */
DllExport ERR __stdcall pa_ext_sim(ARM);	/* Simulation control end (simulation communication end)	*/

/*
 *	Digital input/output function
 */
DllExport ERR __stdcall pa_swt_dio(ARM,INT32);
DllExport ERR __stdcall pa_chg_dio(ARM,DIOSTATUSP);
DllExport ERR __stdcall pa_cpp_dio(ARM,DIOSTATUSP,DIOSTATUSP);
			
/*
 *	Function on teach point operation & playback control
 */
DllExport ERR __stdcall pa_set_idn(ARM,PNTID,INT32);	/* ID_No. setting at teach point				*/
DllExport ERR __stdcall pa_get_idn(ARM,INT32*);		/* Current point ID_No. loading					*/
DllExport ERR __stdcall pa_axs_pnt(ARM,INT32);	/* Axis motion control from the present position to the current point	*/
DllExport ERR __stdcall pa_set_dlc(ARM,INT32);	/* Playback DO automatic stop/non stop setting 				*/
DllExport ERR __stdcall pa_get_dlc(ARM,INT32*);	/* Playback DO automatic stop/non stop loading 			*/

/*
 *	Other functions
 */
DllExport ERR __stdcall   pa_set_wav(ARM,WAVATTR*);	/* Weaving trajectory setting			        */
DllExport ERR __stdcall   pa_odr_xyz(ARM,TRANSMATP);	/* Tip position offset value setting  			*/
DllExport ERR __stdcall   pa_lmt_xyz(ARM,INT32);	/* Limit value setting when in offset value supplement 	*/
DllExport ERR __stdcall   pa_get_lmt(ARM,REAL32*);	/* Limit value loading when in offset value supplement	*/
DllExport INT16 __stdcall pa_fsh_sub(ARM);		/* Waiting for control command processing completion	*/
DllExport ERR __stdcall   pa_req_sub(ARM,INT32);	/* Issuing command setting intrusion to controller	*/

/*
 *	Function on coordinate conversion matrix
 */
DllExport ERR __stdcall pa_set_mat(ARM,MATRIX,MATRIX);

/*
 *	Direct control function
 */
DllExport ERR __stdcall pa_drt_ded(ARM,INT32);	/* Arm installation direction setting			*/
DllExport ERR __stdcall pa_chk_cnt(ARM);	/* Synchronization processing in direct control 	*/
DllExport ERR __stdcall pa_set_tim(ARM,INT32);	/* Time-out setting in synchronization processing 	*/

/*
 *	Arm status information loading function
 */
DllExport ERR __stdcall pa_get_mod(ARM,INT32*);		/* Arm control status loading  				*/
DllExport ERR __stdcall pa_get_jou(ARM,INT32*);		/* Arm redundant axis control mode loading 		*/
DllExport ERR __stdcall pa_get_ver(ARM,REAL32*);	/* Motion controller S/W version number loading   	*/
DllExport ERR __stdcall pa_get_pvl(ARM,REAL32*);	/* Playback control velocity coefficient loading   	*/
DllExport ERR __stdcall pa_get_pdo(ARM,INT32*);		/* Playback control teach point DO valid/invalid loading */
DllExport ERR __stdcall pa_get_pcd(ARM,DIOSTATUSP,DIOSTATUSP);
DllExport ERR __stdcall pa_get_cpt(ARM,PNTNO,PNTDATP);	/* Current point circle/arc teach data loading 		*/
DllExport ERR __stdcall pa_get_mat(ARM,MATRIX,MATRIX);	/* Current setting conversion matrix loading 		*/
DllExport ERR __stdcall pa_get_sns(ARM,TRANSMATP);	/* Current setting tip offset value loading@		*/
DllExport ERR __stdcall pa_get_tar(ARM,ARMTARGETP);	/* Current arm target data loading@			*/
DllExport ERR __stdcall pa_get_com(ARM,INT32*);		/* Communication status (no communication/simulation/ actual machine                                                            loading   	*/
DllExport ERR __stdcall pa_get_tim(ARM,INT32*);		/* Time-out loading in synchronization processing	*/
DllExport ERR __stdcall pa_get_drt(ARM,INT32*);		/* Arm installation direction acquisition/loading	*/
//DllExport ERR __stdcall pa_get_cps(ARM,INT32*);		     @@@@@@@@@@

/*
 *	Redundant axis control function
 */
DllExport ERR __stdcall pa_mov_jou(ARM,REAL32,INT32);	/* Redundant axis (elbow) motion control 	*/
/*
 *	axis real-time control function
 */
DllExport ERR __stdcall pa_mod_axs(ARM);		/* Axis real-time control mode setting	*/
DllExport ERR __stdcall pa_odr_axs(ARM,ANGLEP);		/* Axis real-time control data set 	*/


/*
 *	Other functions pa_mem.c
 */
//DllExport ERR __stdcall pa_mem_cpy(long,void*,void*,size_t,int);@@@@@
DllExport ERR __stdcall pa_cpy_flt(long,void*,void*,int);	
DllExport ERR __stdcall pa_cpy_lng(long,void*,void*,int);	
DllExport ERR __stdcall pa_cpy_uby(long,void*,void*,int);	
DllExport ERR __stdcall pa_ret_lng(long,void*);				
/* pa_egt.c */
DllExport ERR __stdcall	pa_get_lgh(ARM,REAL32*);			
DllExport ERR __stdcall pa_get_dsw(ARM,INT32*);				


DllExport ERR __stdcall WinRTinit(long);
DllExport ERR __stdcall WinRTabort(long);
DllExport ERR __stdcall MakeMutex(long);
DllExport ERR __stdcall DestroyMutex(long);
DllExport ERR __stdcall LockMutex(long);
DllExport ERR __stdcall UnlockMutex(long);

/*
 *
 */
DllExport long __stdcall pa_sta_log(ARM,long,long);
DllExport long __stdcall pa_end_log(ARM);


/*
 *
 */
DllExport long __stdcall rpa_opn_mmf(long);
DllExport long __stdcall rpa_set_mmf(long,float,float,float,float,float,float,float);
DllExport long __stdcall rpa_cls_mmf(long);

#ifdef __cplusplus
}
#endif


#endif	/* _PA_H_	*/
